/**
  ******************************************************************************
  * @file   ble_service.c
  * @author  Central LAB
  * @version V1.0.0
  * @date     30-January-2016
  * @brief   Add one bluetooth console service using vendor specific profile.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2016 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
#include <stdio.h>
#include "main.h"
#include "ble_service.h"
#include "console.h"

/* Exported variables ---------------------------------------------------------*/
int connected = FALSE;
uint8_t set_connectable = TRUE;
uint16_t connection_handle = 0;

uint16_t ConsoleW2STHandle;
uint16_t TermCharHandle;
uint16_t StdErrCharHandle;

uint8_t TrustedAddr[6];
uint8_t CurrentAddr[6];

/* Private macro ------------------------------------------------------------*/
#define COPY_UUID_128(uuid_struct, uuid_15, uuid_14, uuid_13, uuid_12, uuid_11, uuid_10, uuid_9, uuid_8, uuid_7, uuid_6, uuid_5, uuid_4, uuid_3, uuid_2, uuid_1, uuid_0) \
{\
  uuid_struct[0 ] = uuid_0 ; uuid_struct[1 ] = uuid_1 ; uuid_struct[2 ] = uuid_2 ; uuid_struct[3 ] = uuid_3 ; \
  uuid_struct[4 ] = uuid_4 ; uuid_struct[5 ] = uuid_5 ; uuid_struct[6 ] = uuid_6 ; uuid_struct[7 ] = uuid_7 ; \
  uuid_struct[8 ] = uuid_8 ; uuid_struct[9 ] = uuid_9 ; uuid_struct[10] = uuid_10; uuid_struct[11] = uuid_11; \
  uuid_struct[12] = uuid_12; uuid_struct[13] = uuid_13; uuid_struct[14] = uuid_14; uuid_struct[15] = uuid_15; \
}

/* Private define ------------------------------------------------------------*/
/* Console Service */
#define COPY_CONSOLE_SERVICE_UUID(uuid_struct)   COPY_UUID_128(uuid_struct,0x00,0x00,0x00,0x00,0x00,0x0E,0x11,0xe1,0x9a,0xb4,0x00,0x02,0xa5,0xd5,0xc5,0x1b)
#define COPY_TERM_CHAR_UUID(uuid_struct)         COPY_UUID_128(uuid_struct,0x00,0x00,0x00,0x01,0x00,0x0E,0x11,0xe1,0xac,0x36,0x00,0x02,0xa5,0xd5,0xc5,0x1b)
#define COPY_STDERR_CHAR_UUID(uuid_struct)       COPY_UUID_128(uuid_struct,0x00,0x00,0x00,0x02,0x00,0x0E,0x11,0xe1,0xac,0x36,0x00,0x02,0xa5,0xd5,0xc5,0x1b)

/* Imported Variables -------------------------------------------------------------*/

extern uint32_t StdErrNotification;
extern uint32_t TermNotification;

extern uint8_t BufferToWrite[256];
extern int32_t BytesToWrite;

extern uint8_t manuf_data[30];

extern uint8_t bnrg_expansion_board;

extern TIM_HandleTypeDef    TimPingHandle;

/* Private variables ------------------------------------------------------------*/

static uint8_t LastStderrBuffer[W2ST_CONSOLE_MAX_CHAR_LEN];
static uint8_t LastStderrLen;
static uint8_t LastTermBuffer[W2ST_CONSOLE_MAX_CHAR_LEN];
static uint8_t LastTermLen;
static int MasterTrusted = FALSE;

/* Private functions ------------------------------------------------------------*/
static void GAP_ConnectionComplete_CB(uint8_t addr[6], uint16_t handle);
static void GAP_DisconnectionComplete_CB(void);

#ifdef ACC_BLUENRG_CONGESTION
#define ACI_GATT_UPDATE_CHAR_VALUE safe_aci_gatt_update_char_value
static int32_t breath;


/* @brief  Update the value of a characteristic avoiding (for a short time) to
 *         send the next updates if an error in the previous sending has
 *         occurred.
 * @param  servHandle The handle of the service
 * @param  charHandle The handle of the characteristic
 * @param  charValOffset The offset of the characteristic
 * @param  charValueLen The length of the characteristic
 * @param  charValue The pointer to the characteristic
 * @retval tBleStatus Status
 */
tBleStatus safe_aci_gatt_update_char_value(uint16_t servHandle, 
				      uint16_t charHandle,
				      uint8_t charValOffset,
				      uint8_t charValueLen,   
				      const uint8_t *charValue)
{
  tBleStatus ret = BLE_STATUS_INSUFFICIENT_RESOURCES;
  
  if (breath > 0) {
    breath--;
  } else {
    ret = aci_gatt_update_char_value(servHandle,charHandle,charValOffset,charValueLen,charValue);
    
    if (ret != BLE_STATUS_SUCCESS){
      breath = ACC_BLUENRG_CONGESTION_SKIP;
    }
  }
  
  return (ret);
}

#else /* ACC_BLUENRG_CONGESTION */
#define ACI_GATT_UPDATE_CHAR_VALUE aci_gatt_update_char_value
#endif /* ACC_BLUENRG_CONGESTION */


/**
 * @brief  Add the Console service using a vendor specific profile
 * @param  None
 * @retval tBleStatus Status
 */
tBleStatus Add_ConsoleW2ST_Service(void)
{
  tBleStatus ret;

  uint8_t uuid[16];

  COPY_CONSOLE_SERVICE_UUID(uuid);
  ret = aci_gatt_add_serv(UUID_TYPE_128,  uuid, PRIMARY_SERVICE, 1+3*2,&ConsoleW2STHandle);

  if (ret != BLE_STATUS_SUCCESS) {
    goto fail;
  }

  COPY_TERM_CHAR_UUID(uuid);
  ret =  aci_gatt_add_char(ConsoleW2STHandle, UUID_TYPE_128, uuid, W2ST_CONSOLE_MAX_CHAR_LEN,
                           CHAR_PROP_NOTIFY| CHAR_PROP_WRITE_WITHOUT_RESP | CHAR_PROP_WRITE | CHAR_PROP_READ ,
                           ATTR_PERMISSION_NONE,
                           GATT_NOTIFY_ATTRIBUTE_WRITE | GATT_NOTIFY_READ_REQ_AND_WAIT_FOR_APPL_RESP,
                           16, 1, &TermCharHandle);

  if (ret != BLE_STATUS_SUCCESS) {
    goto fail;
  }

  COPY_STDERR_CHAR_UUID(uuid);
  ret =  aci_gatt_add_char(ConsoleW2STHandle, UUID_TYPE_128, uuid, W2ST_CONSOLE_MAX_CHAR_LEN,
                           CHAR_PROP_NOTIFY | CHAR_PROP_READ,
                           ATTR_PERMISSION_NONE,
                           GATT_NOTIFY_READ_REQ_AND_WAIT_FOR_APPL_RESP,
                           16, 1, &StdErrCharHandle);

  if (ret != BLE_STATUS_SUCCESS) {
     goto fail;
  }

  return BLE_STATUS_SUCCESS;

fail:
  //printf("Error while adding Console service.\n");
  return BLE_STATUS_ERROR;
}

/**
 * @brief  Update Stderr characteristic value
 * @param  uint8_t *data string to write
 * @param  uint8_t lenght lengt of string to write
 * @retval tBleStatus      Status
 */
tBleStatus Stderr_Update(uint8_t *data,uint8_t length)
{
  tBleStatus ret;
  uint8_t Offset;
  uint8_t DataToSend;

  /* Split the code in packages*/
  for(Offset =0; Offset<length; Offset +=W2ST_CONSOLE_MAX_CHAR_LEN){
    DataToSend = (length-Offset);
    DataToSend = (DataToSend>W2ST_CONSOLE_MAX_CHAR_LEN) ?  W2ST_CONSOLE_MAX_CHAR_LEN : DataToSend;

    /* keep a copy */
    memcpy(LastStderrBuffer,data+Offset,DataToSend);
    LastStderrLen = DataToSend;

    ret = ACI_GATT_UPDATE_CHAR_VALUE(ConsoleW2STHandle, StdErrCharHandle, 0, DataToSend , data+Offset);
    if (ret != BLE_STATUS_SUCCESS) {
      return BLE_STATUS_ERROR;
    }
  }

  return BLE_STATUS_SUCCESS;
}

/**
 * @brief  Update Terminal characteristic value
 * @param  uint8_t *data string to write
 * @param  uint8_t lenght lengt of string to write
 * @retval tBleStatus      Status
 */
tBleStatus Term_Update(uint8_t *data,uint8_t length)
{
  tBleStatus ret;
  uint8_t Offset;
  uint8_t DataToSend;

  /* Split the code in packages */
  for(Offset =0; Offset<length; Offset +=W2ST_CONSOLE_MAX_CHAR_LEN){
    DataToSend = (length-Offset);
    DataToSend = (DataToSend>W2ST_CONSOLE_MAX_CHAR_LEN) ?  W2ST_CONSOLE_MAX_CHAR_LEN : DataToSend;

    /* keep a copy */
    memcpy(LastTermBuffer,data+Offset,DataToSend);
    LastTermLen = DataToSend;

    ret = ACI_GATT_UPDATE_CHAR_VALUE(ConsoleW2STHandle, TermCharHandle, 0, DataToSend , data+Offset);
    if (ret != BLE_STATUS_SUCCESS) {
      if(StdErrNotification){
        BytesToWrite = sprintf((char *)BufferToWrite, "Error Updating Stdout Char\r\n");
        Stderr_Update(BufferToWrite,BytesToWrite);
      } else {
        printf("Error Updating Stdout Char\r\n");
      }
      return BLE_STATUS_ERROR;
    }
  }

  return BLE_STATUS_SUCCESS;
}

/**
 * @brief  Update Stderr characteristic value after a read request
 * @param None
 * @retval tBleStatus      Status
 */
static tBleStatus Stderr_Update_AfterRead(void)
{
  tBleStatus ret;

  ret = ACI_GATT_UPDATE_CHAR_VALUE(ConsoleW2STHandle, StdErrCharHandle, 0, LastStderrLen , LastStderrBuffer);
  if (ret != BLE_STATUS_SUCCESS) {
    return BLE_STATUS_ERROR;
  }

  return BLE_STATUS_SUCCESS;
}

/**
 * @brief  Update Terminal characteristic value after a read request
 * @param None
 * @retval tBleStatus      Status
 */
static tBleStatus Term_Update_AfterRead(void)
{
  tBleStatus ret;

  ret = ACI_GATT_UPDATE_CHAR_VALUE(ConsoleW2STHandle, TermCharHandle, 0, LastTermLen , LastTermBuffer);
  if (ret != BLE_STATUS_SUCCESS) {
    if(StdErrNotification){
      BytesToWrite = sprintf((char *)BufferToWrite, "Error Updating Stdout Char\r\n");
      Stderr_Update(BufferToWrite,BytesToWrite);
    } else {
      printf("Error Updating Stdout Char\r\n");
    }
    return BLE_STATUS_ERROR;
  }

  return BLE_STATUS_SUCCESS;
}



/**
 * @brief  Puts the device in connectable mode.
 *         If you want to specify a UUID list in the advertising data, those data can
 *         be specified as a parameter in aci_gap_set_discoverable().
 *         For manufacture data, aci_gap_update_adv_data must be called
 * @param  None 
 * @retval None
 */
void setConnectable(void)
{  
  char local_name[8] = {AD_TYPE_COMPLETE_LOCAL_NAME,NAME_NFC}; 
  
  /* disable scan response */
  hci_le_set_scan_resp_data(0,NULL);

  aci_gap_set_discoverable(ADV_IND, 0, 0, PUBLIC_ADDR, NO_WHITE_LIST_USE,
                           sizeof(local_name), local_name, 0, NULL, 0, 0);

  /* Send Advertising data */
  aci_gap_update_adv_data(26, manuf_data);
}

/**
 * @brief  This function is called when there is a LE Connection Complete event.
 * @param  uint8_t addr[6] Address of peer device
 * @param  uint16_t handle Connection handle
 * @retval None
 */
static void GAP_ConnectionComplete_CB(uint8_t addr[6], uint16_t handle)
{  
  connected = TRUE;
  connection_handle = handle;

#ifdef NFC_DEBUG_CONNECTION
  printf(">>>>>>CONNECTED %x:%x:%x:%x:%x:%x\r\n",addr[5],addr[4],addr[3],addr[2],addr[1],addr[0]);
#endif /* NFC_DEBUG_CONNECTION */

  StdErrNotification=0;
  TermNotification  =0;

  MasterTrusted = FALSE;
  
#ifdef NFC_SECURE_CONNECTION
  {
    int i;
    int AlreadyTrusted=1;
    for(i=0;i<6;i++) {
      CurrentAddr[i]= addr[i];
      if(addr[i]!=TrustedAddr[i]){
        AlreadyTrusted=0;
      }
    }
    if(!AlreadyTrusted){
      /* Send a slave security request to the master */
      int ret = aci_gap_slave_security_request(handle, NO_BONDING, MITM_PROTECTION_REQUIRED);
      if (ret != BLE_STATUS_SUCCESS) {
       printf("\r\nGAP Slave secury request failed\r\n");
      }
    } else {
      printf("Device already trusted\r\n");
      MasterTrusted = TRUE;
    }
  }
#else /* NFC_SECURE_CONNECTION */
  MasterTrusted = TRUE;
#endif /* NFC_SECURE_CONNECTION */

  /* Start the TIM Base generation in interrupt mode */
  if(HAL_TIM_Base_Start_IT(&TimPingHandle) != HAL_OK){
    /* Starting Error */
    Error_Handler();
  }
}

/**
 * @brief  This function is called when the peer device get disconnected.
 * @param  None 
 * @retval None
 */
static void GAP_DisconnectionComplete_CB(void)
{
  connected = FALSE;

#ifdef NFC_DEBUG_CONNECTION  
  printf("<<<<<<DISCONNECTED\r\n");
#endif /* NFC_DEBUG_CONNECTION */  

  /* Make the device connectable again. */
  set_connectable = TRUE;

  StdErrNotification=0;
  TermNotification  =0;

  MasterTrusted = FALSE;
  
  /* Stop the TIM Base generation in interrupt mode */
  if(HAL_TIM_Base_Stop_IT(&TimPingHandle) != HAL_OK){
    /* Stopping Error */
    Error_Handler();
  }
}

/**
 * @brief  This function is called when there is a Bluetooth Read request
 * @param  uint16_t handle Handle of the attribute
 * @retval None
 */
void Read_Request_CB(uint16_t handle)
{
  if (handle == StdErrCharHandle + 1) {
    /* Send again the last packet for StdError */
    Stderr_Update_AfterRead();
  } else if (handle == TermCharHandle + 1) {
    /* Send again the last packet for Terminal */
    Term_Update_AfterRead();
  }

  //EXIT:
  if(connection_handle != 0)
    aci_gatt_allow_read(connection_handle);
}

/**
 * @brief  This function is called when there is a change on the gatt attribute
 * With this function it's possible to understand if one application 
 * is subscribed or not to the one service
 * @param uint16_t att_handle Handle of the attribute
 * @param uint8_t *att_data attribute data
 * @param uint8_t data_length length of the data
 * @retval None
 */
void Attribute_Modified_CB(uint16_t attr_handle, uint8_t * att_data, uint8_t data_length) {
  if(attr_handle == StdErrCharHandle + 2){
    if (att_data[0] == 01) {
      StdErrNotification=1;
    } else if (att_data[0] == 0){
      StdErrNotification=0;
    }
#ifdef NFC_DEBUG_CONNECTION
    if(TermNotification) {
      BytesToWrite = sprintf((char *)BufferToWrite,"--->StdE=%s\r\n", StdErrNotification ? "ON" : "OFF");
     Term_Update(BufferToWrite,BytesToWrite);
    } else
      printf("--->StdE=%s\r\n", StdErrNotification ? "ON" : "OFF");
#endif /* NFC_DEBUG_CONNECTION */
  } else if(attr_handle == TermCharHandle + 2){
    if (att_data[0] == 01) {
      TermNotification=1;
    } else if (att_data[0] == 0){
      TermNotification=0;
    }
#ifdef NFC_DEBUG_CONNECTION
    printf("--->Term=%s\r\n", TermNotification ? "ON" : "OFF");
#endif /* NFC_DEBUG_CONNECTION */
  } else if (attr_handle == TermCharHandle + 1){
    /* Received one write from Client on Terminal characteristc */
    /* Send it back for testing */
    Term_Update(att_data,data_length);
  } else {
    if(StdErrNotification){
      BytesToWrite = sprintf((char *)BufferToWrite, "Notification UNKNOW handle\r\n");
      Stderr_Update(BufferToWrite,BytesToWrite);
    } else {
      printf("Notification UNKNOW handle\r\n");
    }
  }
}

/**
 * @brief  This function is called whenever there is an ACI event to be processed.
 * @note   Inside this function each event must be identified and correctly
 *         parsed.
 * @param  void *pckt Pointer to the ACI packet
 * @retval None
 */
void HCI_Event_CB(void *pckt)
{
  hci_uart_pckt *hci_pckt = pckt;
  hci_event_pckt *event_pckt = (hci_event_pckt*)hci_pckt->data;
  
  if(hci_pckt->type != HCI_EVENT_PKT) {
    return;
  }
  
  switch(event_pckt->evt){
    
  case EVT_DISCONN_COMPLETE:
    {
      GAP_DisconnectionComplete_CB();
    }
    break;
  case EVT_LE_META_EVENT:
    {
      evt_le_meta_event *evt = (void *)event_pckt->data;
      
      switch(evt->subevent){
      case EVT_LE_CONN_COMPLETE:
        {
          evt_le_connection_complete *cc = (void *)evt->data;
          GAP_ConnectionComplete_CB(cc->peer_bdaddr, cc->handle);
        }
        break;
      case EVT_LE_CONN_UPDATE_COMPLETE:
        {
          evt_le_connection_update_complete * con_update = (evt_le_connection_update_complete *) evt->data;
          printf("EVT_LE_CONN_UPDATE_COMPLETE status=%d\r\n",con_update->status);
        }
        break;
      default:
        printf("EVT_LE_META_EVENT =%d\r\n",evt->subevent);        
      }
    }
    break;
  case EVT_VENDOR:
    {
      evt_blue_aci *blue_evt = (void*)event_pckt->data;
      switch(blue_evt->ecode){
      case EVT_BLUE_GATT_READ_PERMIT_REQ:
        {
          if(MasterTrusted==TRUE) {
            /* Only if the Master is trusted */
            evt_gatt_read_permit_req *pr = (void*)blue_evt->data;
            Read_Request_CB(pr->attr_handle);
          }
        }
        break;
      case EVT_BLUE_GATT_ATTRIBUTE_MODIFIED:
        {
          if(MasterTrusted==TRUE) {
            /* Only if the Master is trusted */
            if(bnrg_expansion_board==IDB05A1) {
              evt_gatt_attr_modified_IDB05A1 *evt = (evt_gatt_attr_modified_IDB05A1*)blue_evt->data;
              Attribute_Modified_CB(evt->attr_handle, evt->att_data,evt->data_length);
            } else {
              evt_gatt_attr_modified_IDB04A1 *evt = (evt_gatt_attr_modified_IDB04A1*)blue_evt->data;
              Attribute_Modified_CB(evt->attr_handle, evt->att_data,evt->data_length);
            }
          }
        }
        break;
      case EVT_BLUE_GAP_AUTHORIZATION_REQUEST:
        printf("EVT_BLUE_GAP_AUTHORIZATION_REQUEST\r\n");
        break;
      case EVT_BLUE_GAP_PASS_KEY_REQUEST: 
        {
          printf("EVT_BLUE_GAP_PASS_KEY_REQUEST\r\n");
        }
        break;
      case EVT_BLUE_GAP_PAIRING_CMPLT:
        {
          evt_gap_pairing_cmplt *pairing = (evt_gap_pairing_cmplt *)blue_evt->data;
         printf("EVT_BLUE_GAP_PAIRING_CMPLT status=%d\r\n",pairing->status);
         if(pairing->status==SM_PAIRING_SUCCESS) {
           int i;
           MasterTrusted=TRUE;
           for(i=0;i<6;i++) {
             TrustedAddr[i]= CurrentAddr[i];
           }
         }
        }
        break;
      case EVT_BLUE_GAP_SLAVE_SECURITY_INITIATED:
        {
          printf("EVT_BLUE_GAP_SLAVE_SECURITY_INITIATED\r\n");
        }
        break;
      case EVT_BLUE_GAP_BOND_LOST:
        if(bnrg_expansion_board==IDB05A1) {
          int ret;
          evt_gatt_procedure_timeout *evt = (evt_gatt_procedure_timeout*)blue_evt->data;
          printf("EVT_BLUE_GAP_BOND_LOST\r\n");
          ret = aci_gap_allow_rebond_IDB05A1(evt->conn_handle);
          if (ret != BLE_STATUS_SUCCESS) {
            printf("\r\nGAP allow Rebound failed\r\n");
          }
        } else {
          int ret;
          printf("EVT_BLUE_GAP_BOND_LOST\r\n");
          ret = aci_gap_allow_rebond_IDB04A1();
          if (ret != BLE_STATUS_SUCCESS) {
            printf("\r\nGAP allow Rebound failed\r\n");
          }
        }
        break;
      default:
        printf("EVT_Vendor =%d\r\n",blue_evt->ecode);
      }
    }
    break;
  case EVT_ENCRYPT_CHANGE:
    {
      evt_encrypt_change *encrypt = (evt_encrypt_change *)event_pckt->data;
      printf("EVT_ENCRYPT_CHANGE! status=%d\r\n",encrypt->status);
    }
    break;
    default:
        printf("default =%d\r\n",event_pckt->evt);
  }
}
/******************* (C) COPYRIGHT 2016 STMicroelectronics *****END OF FILE****/
