/**
  ******************************************************************************
  * @file    main.c
  * @author  Central Lab
  * @version V1.0.0
  * @date    30-January-2016
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2016 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/**
 * @mainpage FP-SEC-BLENFC1 Bluetooth Low Energy and NFC Pairing Software
 *
 * @image html st_logo.png
 *
 * <b>Introduction</b>
 *
 * This firmware package includes Components Device Drivers, Board Support Package
 * and example application for the following STMicroelectronics elements:
 * - X-NUCLEO-IDB04A1/X-NUCLEO-IDB05A1 Bluetooth Low energy expansion boards
 * - X-NUCLEO-NFC01A1 Dynamic NFC tag expansion board based on M24SR
 * - NUCLEO-F401RE/NUCLEO-L053R8 Nucleo boards
 *
 * <b>Example Application</b>
 *
 * The Example application initizializes all the Components and Libraries.
 * Then at the beginning it writes one URI to M24SR NFC, and after pressing the user button,
 * it starts the Bluetooth stack creating one service that expones the Console BLE characteristic and 
 * writes on the M24SR NFC the information for making facility&Secure BLE pairing.
 * Moreover, On F401RE nucleo board, when there are not device connected with BLE, pressing 
 * the user button is possible to write on NFC alternatively:
 * - the BLE pairing informations
 * - One email that sends informations about the latest device that have tried the connection
 *
 * Example application allows the user to control the initialization phase via UART.
 * Launch a terminal application and set the UART port to 115200 bps, 8 bit, No Parity,
 * 1 stop bit.
 *
 * This example must be used with the related BlueMS Android/iOS application available on Play/itune store
 * in order to read the sent information by Bluetooth Low Energy protocol
 *
 */

/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include <math.h>
#include <limits.h>
#include "main.h"
#include "ble_service.h"
#include "bluenrg_utils.h"

#ifdef USE_STM32F4XX_NUCLEO 
#include "stm32f4xx_UART.h"
#include "stm32f4xx_I2C.h"
#include "stm32f4xx_SPI.h" 
#include "stm32f4xx_periph_conf.h"
#endif /* USE_STM32F4XX_NUCLEO */

#ifdef USE_STM32L0XX_NUCLEO 
#include "stm32l0xx_UART.h"
#include "stm32l0xx_I2C.h"
#include "stm32l0xx_SPI.h" 
#include "stm32l0xx_periph_conf.h"
#endif /* USE_STM32L0XX_NUCLEO */


#include "lib_TagType4.h"
#include "lib_NDEF.h"

/* Private define ------------------------------------------------------------*/

#define BLE_RECORD_SIZE 26
#define BLE_PAYLOAD_SIZE 30

#ifdef USE_STM32F4XX_NUCLEO
#define STM32_UUID ((uint32_t *)0x1FFF7A10)
#endif /* USE_STM32F4XX_NUCLEO */

#ifdef USE_STM32L0XX_NUCLEO
#define STM32_UUID ((uint32_t *)0x1FF80050)
#endif /* USE_STM32L0XX_NUCLEO */

/* Imported Variables -------------------------------------------------------------*/
extern uint8_t set_connectable;
extern int connected;

extern uint8_t TrustedAddr[6];
extern uint8_t CurrentAddr[6];

/* Exported Variables -------------------------------------------------------------*/
uint32_t StdErrNotification =0;
uint32_t TermNotification   =0;

uint8_t BufferToWrite[256];
int32_t BytesToWrite;

uint8_t manuf_data[30] = {
  2  /* lenght*/,0x0A,0x00 /* 0 dBm */, // Trasmission Power
  8  /* lenght*/,0x09,NAME_NFC, // Complete Name
  13 /* lenght*/ ,0xFF,0x01,/*SKD version */
         0x80, /* NUCLEO-Board */
         0x00 /* Prox+Lux */,
         0x00 /* ACC+Gyro+Mag+Pres+Hum+Temp */,
         0x00 /* SensorFusionShort */,
         0x00, /* SensorFusionFloat */
         0x00, /* BLE MAC start */
         0x00,
         0x00,
         0x00,
         0x00,
         0x00, /* BLE MAC stop */
         0x00, /* BLE PIN start */
         0x00,
         0x00,
         0x00  /* BLE PIN stop */
};

uint8_t bnrg_expansion_board = IDB04A1; /* IDB04A1 by default */

TIM_HandleTypeDef    TimPingHandle;
volatile int SendPing=0;

/* Private variables ---------------------------------------------------------*/

#ifdef MAC_NFC
static uint8_t SERVER_BDADDR[6] = {MAC_NFC};
#else /* MAC_NFC */
static uint8_t SERVER_BDADDR[6];
#endif /* MAC_NFC */
static uint32_t PinForParing;

/* NDEF table for BLE pairing */
uint8_t NDEF_BT_PAIRING[BLE_RECORD_SIZE+BLE_PAYLOAD_SIZE+5];

/* Private function prototypes -----------------------------------------------*/
static void SystemClock_Config(void);
static void InitTimer(void);
static void ButtonCallback(void);
static void UserFunction(void);

static void Init_BlueNRG_Custom_Service(void);
static void Init_BlueNRG_Stack(void);

static void Init_M24SR(void);
static void BLE_Preparing_NDEF(void);
static void BLE_Pairing_M24SR(void);
static void initLED(void);
#ifdef USE_STM32F4XX_NUCLEO
static void Write_Email_M24SR(void);
#endif /* USE_STM32L0XX_NUCLEO */

static volatile int ButtonPressed=0;
static volatile int HCI_ProcessEvent=0;

static int StartBlueNRG=0;

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void)
{
  HAL_Init();
  
  /* Configure the System clock */
  SystemClock_Config();
	  
  /* Initialize Nucleo LED */
  BSP_LED_Init(LED2);
  
  /* Initialize the 3 LEDs on NFC expansion board */
  initLED();
  
  /* HUART Global Initialization */
  if(UART_Global_Init()!=HAL_OK) {
    Error_Handler();
  } else {
    printf("UART Initialized\r\n");
  }
  
  /* I2C Global Initialization */
  if(I2C_Global_Init()!=HAL_OK) {
    Error_Handler();
  } else {
    printf("I2C  Initialized\r\n");
  }
  
  /* Banner */
  printf("\r\nSTMicroelectronics FP-SEC-BLENFC1:\r\n"
#ifdef USE_STM32F4XX_NUCLEO
        "\tSTM32F401RE-Nucleo board"
#endif /* USE_STM32F4XX_NUCLEO */
#ifdef USE_STM32L0XX_NUCLEO
        "\tSTM32L053R8-Nucleo board"        
#endif /* USE_STM32L0XX_NUCLEO */
        " (HAL %ld.%ld.%ld_%ld)\r\n"
        "\tCompiled %s %s"
#if defined (__IAR_SYSTEMS_ICC__)
        " (IAR)\r\n"
#elif defined (__CC_ARM)
        " (KEIL)\r\n"
#elif defined (__GNUC__)
        " (openstm32)\r\n"
#endif
          ,HAL_GetHalVersion() >>24,
          (HAL_GetHalVersion() >>16)&0xFF,
          (HAL_GetHalVersion() >> 8)&0xFF,
           HAL_GetHalVersion()      &0xFF,
           __DATE__,__TIME__);
  
  printf("Press the User button for starting the BlueNRG\r\n");

  /* Initialize the M24SR component and write the URI */
  Init_M24SR();
 
  /* Initialize user button */
  BSP_PB_Init(BUTTON_KEY, BUTTON_MODE_EXTI);
  
  /* initialize timer */
  InitTimer();

  /* Infinite loop */
  while (1){

    /* Handle user button */
    if(ButtonPressed) {
      ButtonCallback();
      ButtonPressed=0;
    }

    /* handle BLE event */
    if(HCI_ProcessEvent) {
      HCI_ProcessEvent=0;
      HCI_Process();
    }
    
    if(StartBlueNRG) {
      if(set_connectable){
        setConnectable();
        set_connectable = FALSE;
      }
    }
    
    if(SendPing) {
     UserFunction();
     SendPing=0;
    }

    /* Wait for Event */
    __WFI();    
  }
}

/**
  * @brief  User defined callback when the BLE is connected
  * @param  None
  * @retval None
  */
static void UserFunction(void)
{
  static int32_t LedNum=0;
  switch(LedNum){
  case 0:
    HAL_GPIO_WritePin(GPIOB,GPIO_PIN_4,GPIO_PIN_SET);
    HAL_GPIO_WritePin(GPIOB,GPIO_PIN_5,GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOA,GPIO_PIN_10,GPIO_PIN_RESET);
    LedNum++;
    break;
  case 1:
    HAL_GPIO_WritePin(GPIOB,GPIO_PIN_4,GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOB,GPIO_PIN_5,GPIO_PIN_SET);
    HAL_GPIO_WritePin(GPIOA,GPIO_PIN_10,GPIO_PIN_RESET);
    LedNum++;
    break;
  case 2:
    HAL_GPIO_WritePin(GPIOB,GPIO_PIN_4,GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOB,GPIO_PIN_5,GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOA,GPIO_PIN_10,GPIO_PIN_SET);
    LedNum=0;
   break;
  }
  if(TermNotification) {
     BytesToWrite = sprintf((char *)BufferToWrite,"I'm sending..%ld\r\n",(HAL_GetTick()>>3));
     Term_Update(BufferToWrite,BytesToWrite);
  } else {
    printf("I'm printing..%ld\r\n",(HAL_GetTick()>>3));
  }
 }
/**
  * @brief  Callback for user button
  * @param  None
  * @retval None
  */
static void ButtonCallback(void)
{
#ifdef USE_STM32F4XX_NUCLEO
  static int32_t SwapFunction=0;
#endif /* USE_STM32F4XX_NUCLEO */

  if(StartBlueNRG) {
    /* If the BlueNRG service are started */
#ifdef USE_STM32F4XX_NUCLEO
    if(connected==FALSE) {
      if(SwapFunction) {
        /* Write Log access info */
        Write_Email_M24SR();
        SwapFunction=0;
        /* Switch off LED2 */
        BSP_LED_Off(LED2);
      } else {
        SwapFunction=1;
        /* Write BLE Pairing Information */
        BLE_Pairing_M24SR();
        /* Switch on the LED2 */
        BSP_LED_On(LED2);
      }
    }
#endif /* USE_STM32F4XX_NUCLEO */
  } else {
    /* If the BlueNRG service are not yet started */
    StartBlueNRG=1;
#ifdef USE_STM32F4XX_NUCLEO
    SwapFunction=0;
#endif /* USE_STM32F4XX_NUCLEO */

    /* Switch on the LED2 */
    BSP_LED_On(LED2);

#ifdef NFC_DEBUG_CONNECTION
    printf("Debug Connection         Enabled\r\n");
#endif /* NFC_DEBUG_CONNECTION */

#ifdef NFC_DEBUG_NOTIFY_TRAMISSION
    printf("Debug Notify Trasmission Enabled\r\n");
#endif /* NFC_DEBUG_NOTIFY_TRAMISSION */
    
    /* Initialize the BlueNRG */
    Init_BlueNRG_Stack();

    /* Initialize the BlueNRG Custom */
    Init_BlueNRG_Custom_Service();        

    /* Set output power level */
    aci_hal_set_tx_power_level(1,4);

    /* Preparing Bluetooth NDEF Table for M24SR */
    BLE_Preparing_NDEF();

    /* Write BLE Pairing Information */
    BLE_Pairing_M24SR();
  }
}

/**
 * @brief  Function for initializing one timer for sending
 * a ping value to BLE
 * @param  None
 * @retval None
 */
static void InitTimer(void)
{
  uint32_t uwPrescalerValue;

  /* Compute the prescaler value to have TIM3 counter clock equal to 10 KHz */
  uwPrescalerValue = (uint32_t) ((SystemCoreClock / 10000) - 1);

  /* Set TIM3 instance (Motion)*/
#ifdef USE_STM32F4XX_NUCLEO 
  TimPingHandle.Instance = TIM3;
#endif /* USE_STM32F4XX_NUCLEO */

#ifdef USE_STM32L0XX_NUCLEO
  TimPingHandle.Instance = TIM6;
#endif /* USE_STM32L0XX_NUCLEO */
  /* Initialize TIM3 peripheral as follow:
     + Period = 2s - 1 (10ms)
     + Prescaler = ((SystemCoreClock/2)/10000) - 1
     + ClockDivision = 0
     + Counter direction = Up
  */
  TimPingHandle.Init.Period = 20000 - 1;
  TimPingHandle.Init.Prescaler = uwPrescalerValue;
  TimPingHandle.Init.ClockDivision = 0;
  TimPingHandle.Init.CounterMode = TIM_COUNTERMODE_UP;
  if(HAL_TIM_Base_Init(&TimPingHandle) != HAL_OK) {
    /* Initialization Error */
    Error_Handler();
  }
}

/** @brief Initialize the M24SR component and write URI
 * @param None
 * @retval None
 */
static void Init_M24SR(void)
{
  sURI_Info URI = {
    URI_ID_0x01_STRING,
    "st.com/stm32ode",
    "\0"};
  
  // Init of the Type Tag 4 component (M24SR)
  while (TT4_Init() != SUCCESS)
    ;
  printf("M24SR Initialized\r\n");
  
  while (TT4_WriteURI(&URI) != SUCCESS)
    ;
  printf("M24SR URI written =\"www.st.com/stm32ode\"\r\n");
  
  /* Switch on all the 3 Leds on M24SR */
  HAL_GPIO_WritePin(GPIOB,GPIO_PIN_4,GPIO_PIN_SET);
  HAL_GPIO_WritePin(GPIOB,GPIO_PIN_5,GPIO_PIN_SET);
  HAL_GPIO_WritePin(GPIOA,GPIO_PIN_10,GPIO_PIN_SET);
}

/** @brief Write the NDEF values for NFC BLE pairing
 * @param None
 * @retval None
 */
static void BLE_Pairing_M24SR(void)
{
  /* Write Bluetooth NDEF Table in M24SR */
  while(TT4_WriteNDEF(NDEF_BT_PAIRING)!=SUCCESS)
    ;
  
  printf("M24SR Bluetooth NDEF Table written\r\n");
  
  /* Switch off all the 3 Leds on M24SR */
  HAL_GPIO_WritePin(GPIOB,GPIO_PIN_4,GPIO_PIN_RESET);
  HAL_GPIO_WritePin(GPIOB,GPIO_PIN_5,GPIO_PIN_RESET);
  HAL_GPIO_WritePin(GPIOA,GPIO_PIN_10,GPIO_PIN_RESET);
}

#ifdef USE_STM32F4XX_NUCLEO
/** @brief Write the Log access info email in NFC
 * @param None
 * @retval None
 */
static void Write_Email_M24SR(void)
{
  /* Field to fill for the Email demonstration */
  sEmailInfo EmailStruct = {
    "test.email@st.com\0", //EmailAdd
    "FP-SEC-BLENFC1 (Nucleo + NFC + BLE)\0", //Subject
    "\0", //Message body below
    "Connection Log\0" //Information
  };
#ifdef NFC_SECURE_CONNECTION
  sprintf((char *)EmailStruct.Message,"Latest Device %x:%x:%x:%x:%x:%x Device Trusted %x:%x:%x:%x:%x:%x",
    CurrentAddr[5],CurrentAddr[4],CurrentAddr[3],CurrentAddr[2],CurrentAddr[1],CurrentAddr[0],
    TrustedAddr[5],TrustedAddr[4],TrustedAddr[3],TrustedAddr[2],TrustedAddr[1],TrustedAddr[0]);
#else /* NFC_SECURE_CONNECTION */
  sprintf((char *)EmailStruct.Message,"Secure pairing not enabled, define NFC_SECURE_CONNECTION on Inc/nfc_config.h file");
#endif /* NFC_SECURE_CONNECTION */

  while(TT4_WriteEmail (&EmailStruct)!=SUCCESS)
    ;
  printf("M24SR Email NDEF written\r\n");
}
#endif /* USE_STM32F4XX_NUCLEO */

/** @brief Preparing the NDEF values for NFC BLE pairing
 * @param None
 * @retval None
 */
static void BLE_Preparing_NDEF(void)
{
  uint8_t NDEF_APP[BLE_RECORD_SIZE]={"application/com.st.W2STApp"};
  
   /* Fill The advertizing table */
  manuf_data[20] = SERVER_BDADDR[5];
  manuf_data[21] = SERVER_BDADDR[4];
  manuf_data[22] = SERVER_BDADDR[3];
  manuf_data[23] = SERVER_BDADDR[2];
  manuf_data[24] = SERVER_BDADDR[1];
  manuf_data[25] = SERVER_BDADDR[0];
  
  manuf_data[26] =  PinForParing     &0xFF;
  manuf_data[27] = (PinForParing>>8 )&0xFF;
  manuf_data[28] = (PinForParing>>16)&0xFF;
  manuf_data[29] = (PinForParing>>24)&0xFF;

  /* NLEN; NDEF length  byte long message) */
  NDEF_BT_PAIRING[0] = 0x00;
  NDEF_BT_PAIRING[1] = BLE_RECORD_SIZE+BLE_PAYLOAD_SIZE+3;
  NDEF_BT_PAIRING[2] = 0xD2; //MB=1b, ME=1b, CF=0b, SR=1b, IL=0b, TNF=010b
  NDEF_BT_PAIRING[3] = BLE_RECORD_SIZE; //Record Type Length [octets]
  NDEF_BT_PAIRING[4] = BLE_PAYLOAD_SIZE; //payload length [octets]
  memcpy(NDEF_BT_PAIRING+5,NDEF_APP,BLE_RECORD_SIZE);
  memcpy(NDEF_BT_PAIRING+BLE_RECORD_SIZE+5,manuf_data,BLE_PAYLOAD_SIZE);
}

/** @brief Initialize the BlueNRG Stack
 * @param None
 * @retval None
 */
static void Init_BlueNRG_Stack(void)
{
  const char BoardName[8] = {NAME_NFC,0};
  uint16_t service_handle, dev_name_char_handle, appearance_char_handle;
  int ret;
  uint8_t  hwVersion;
  uint16_t fwVersion;
  
  /* Initialize the BlueNRG SPI driver */
  if(SPI_Global_Init()!=HAL_OK) {
    Error_Handler();
  } else {
    printf("SPI Initialized\r\n");
  }
  
  /* Initialize the BlueNRG HCI */
  HCI_Init();
  
  /* Reset BlueNRG hardware */
  BlueNRG_RST();
  
  /* get the BlueNRG HW and FW versions */
  getBlueNRGVersion(&hwVersion, &fwVersion);

  if (hwVersion > 0x30) {
    /* X-NUCLEO-IDB05A1 expansion board is used */
    bnrg_expansion_board = IDB05A1;
  } else {
    /* X-NUCLEO-IDB0041 expansion board is used */
    bnrg_expansion_board = IDB04A1;
  }
  
  /* 
   * Reset BlueNRG again otherwise we are not
   * be able to change its MAC address.
   * aci_hal_write_config_data() must be the first
   * command after reset otherwise it will fail.
   */
  BlueNRG_RST();
  
#if ((defined NFC_SECURE_CONNECTION) || (!defined MAC_NFC))
  /* initialize random seed: */
  srand (STM32_UUID[0] + STM32_UUID[1] + STM32_UUID[2] + hwVersion*fwVersion);
#endif /* ((defined NFC_SECURE_CONNECTION) || (!defined MAC_NFC)) */
  
#ifndef MAC_NFC
  /* Create Random BLE MAC address */
  {
    int i;

    for(i=0;i<6;i++) {
      SERVER_BDADDR[i] = rand()&0xFF;
    }
  }
#endif /* MAC_NFC */

#ifdef NFC_SECURE_CONNECTION
    /* Create Random Pin for Pairing */
    /* Whole 6 digit pin */
    PinForParing = rand()&0x7FFFF;
#endif /* NFC_SECURE_CONNECTION */

  ret = aci_hal_write_config_data(CONFIG_DATA_PUBADDR_OFFSET,
                                  CONFIG_DATA_PUBADDR_LEN,
                                  SERVER_BDADDR);
  if(ret){
     printf("\r\nSetting BD_ADDR failed\r\n");
     goto fail;
  }
  
  ret = aci_gatt_init();    
  if(ret){
     printf("\r\nGATT_Init failed\r\n");
     goto fail;
  }
  
  if (bnrg_expansion_board == IDB05A1) {
    ret = aci_gap_init_IDB05A1(GAP_PERIPHERAL_ROLE_IDB05A1, 0, 0x07, &service_handle, &dev_name_char_handle, &appearance_char_handle);
  }else {
    ret = aci_gap_init_IDB04A1(GAP_PERIPHERAL_ROLE_IDB04A1, &service_handle, &dev_name_char_handle, &appearance_char_handle);
  }

        
  if(ret != BLE_STATUS_SUCCESS){
     printf("\r\nGAP_Init failed\r\n");
     goto fail;
  }

  ret = aci_gatt_update_char_value(service_handle, dev_name_char_handle, 0,
                                   7/*strlen(BoardName)*/, (uint8_t *)BoardName);

  if(ret){
     printf("\r\naci_gatt_update_char_value failed\r\n");
    while(1);
  }

  ret=aci_gap_clear_security_database();
  if (ret != BLE_STATUS_SUCCESS) {
     printf("\r\nGAP clear security database failed\r\n");
     goto fail;
  }
  
  ret = aci_gap_set_auth_requirement(MITM_PROTECTION_REQUIRED,
                                     OOB_AUTH_DATA_ABSENT,
                                     NULL, 7, 16,
                                     USE_FIXED_PIN_FOR_PAIRING, PinForParing,
                                     NO_BONDING);
  if (ret != BLE_STATUS_SUCCESS) {
     printf("\r\nGAP setting Authentication failed\r\n");
     goto fail;
  }
   printf("SERVER: BLE Stack Initialized \r\n"
          "\t\tBoard type=%s HWver=%d, FWver=%d\r\n"
          "\t\tBoardName= %s\r\n"
          "\t\tBoardMAC = %x:%x:%x:%x:%x:%x\r\n"
          "\t\tPin=%06lu\r\n\n",
         (bnrg_expansion_board==IDB05A1) ? "IDB05A1" : "IDB04A1",
         hwVersion, fwVersion,
         BoardName, 
         SERVER_BDADDR[5],SERVER_BDADDR[4],SERVER_BDADDR[3],SERVER_BDADDR[2],SERVER_BDADDR[1],SERVER_BDADDR[0],
         PinForParing);
#ifdef NFC_SECURE_CONNECTION
   printf("\t\tOnly Secure connection allowed\r\n\n");
#endif /* NFC_SECURE_CONNECTION */   
  return;
  
fail:
  return;
}
  
/** @brief Initialize the Custom Console BlueNRG service
 * @param None
 * @retval None
 */
static void Init_BlueNRG_Custom_Service(void)
{
  int ret;
  
  ret = Add_ConsoleW2ST_Service();

  if(ret == BLE_STATUS_SUCCESS)
     printf("Console Service W2ST added successfully\r\n");
  else
     printf("\r\nError while adding Console Service W2ST\r\n");
}

#ifdef USE_STM32F4XX_NUCLEO
/**
  * @brief  System Clock Configuration
  *         The system Clock is configured as follow:
  *            System Clock source            = PLL (HSI)
  *            SYSCLK(Hz)                     = 84000000
  *            HCLK(Hz)                       = 84000000
  *            AHB Prescaler                  = 1
  *            APB1 Prescaler                 = 2
  *            APB2 Prescaler                 = 1
  *            HSI Frequency(Hz)              = 16000000
  *            PLL_M                          = 16
  *            PLL_N                          = 336
  *            PLL_P                          = 4
  *            PLL_Q                          = 7
  *            VDD(V)                         = 3.3
  *            Main regulator output voltage  = Scale2 mode
  *            Flash Latency(WS)              = 2
  * @param  None
  * @retval None
  */
static void SystemClock_Config(void)
{
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_OscInitTypeDef RCC_OscInitStruct;

  /* Enable Power Control clock */
  __PWR_CLK_ENABLE();

  /* The voltage scaling allows optimizing the power consumption when the device is
     clocked below the maximum system frequency, to update the voltage scaling value
     regarding system frequency refer to product datasheet.  */
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE2);

  /* Enable HSI Oscillator and activate PLL with HSI as source */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = 0x10;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
  RCC_OscInitStruct.PLL.PLLM = 16;
  RCC_OscInitStruct.PLL.PLLN = 336;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV4;
  RCC_OscInitStruct.PLL.PLLQ = 7;
  if(HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK){
    Error_Handler();
  }

  /* Select PLL as system clock source and configure the HCLK, PCLK1 and PCLK2 clocks dividers */
  RCC_ClkInitStruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2);
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;
  if(HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK){
    Error_Handler();
  }
}
#endif /* USE_STM32F4XX_NUCLEO */

#ifdef USE_STM32L0XX_NUCLEO
/**
  * @brief  System Clock Configuration
  *         The system Clock configuration
  * @param  None
  * @retval None
  */
static void SystemClock_Config(void)
{
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_OscInitTypeDef RCC_OscInitStruct;
  __PWR_CLK_ENABLE();
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = 0x10;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLLMUL_4;
  RCC_OscInitStruct.PLL.PLLDIV = RCC_PLLDIV_2;
  HAL_RCC_OscConfig(&RCC_OscInitStruct);

  RCC_ClkInitStruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2);
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;
  HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_1);

  __SYSCFG_CLK_ENABLE();
}
#endif /* USE_STM32L0XX_NUCLEO */

/**
  * @brief  Intitialize the leds of the M24SR expansion board
  * @param  None
  * @retval None
  */
static void initLED(void)
{
  GPIO_InitTypeDef GPIO_InitStruct;
  __GPIOA_CLK_ENABLE();
  __GPIOB_CLK_ENABLE();
  /* Configure GPIO for LEDs (PB4,PB5,PA10)*/
  GPIO_InitStruct.Pin = GPIO_PIN_4 | GPIO_PIN_5;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
 #ifdef USE_STM32F4XX_NUCLEO	
  GPIO_InitStruct.Speed = GPIO_SPEED_FAST;
#elif (defined USE_STM32L0XX_NUCLEO)
  GPIO_InitStruct.Speed = GPIO_SPEED_HIGH;
#endif  
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
  
  GPIO_InitStruct.Pin = GPIO_PIN_10;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
}

/**
  * @brief This function provides accurate delay (in milliseconds) based 
  *        on variable incremented.
  * @note In the default implementation , SysTick timer is the source of time base.
  *       It is used to generate interrupts at regular time intervals where uwTick
  *       is incremented.
  * @note This is a user file implementation using WFI state
  * @param Delay: specifies the delay time length, in milliseconds.
  * @retval None
  */
void HAL_Delay(__IO uint32_t Delay)
{
  uint32_t tickstart = 0;
  tickstart = HAL_GetTick();
  while((HAL_GetTick() - tickstart) < Delay){
    __WFI();
  }
}

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
void Error_Handler(void)
{
  /* User may add here some code to deal with this error */
  while(1){
  }
}

/**
 * @brief  EXTI line detection callback.
 * @param  uint16_t GPIO_Pin Specifies the pins connected EXTI line
 * @retval None
 */
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
  switch(GPIO_Pin){
    case SPI1_CMN_DEFAULT_IRQ_PIN:
      HCI_Isr();
      HCI_ProcessEvent=1;
    break;
  case KEY_BUTTON_PIN:
    ButtonPressed = 1;
    break;
  }
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1){
  }
}
#endif

/******************* (C) COPYRIGHT 2016 STMicroelectronics *****END OF FILE****/
